part of '../boxw.dart';

class BoxWInput extends StatelessWidget {
  const BoxWInput({
    super.key,
    required this.title,
    required this.onChanged,
    this.initial,
    this.controller,
    this.validator,
    this.leadingButton,
    this.trailingButton,
    this.prefixIcon,
    this.suffixIcon,
    this.obscureText = false,
    this.keyboardType,
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.style,
    this.titleStyle,
    this.minLines = 1,
    this.maxLines,
    this.radius,
    this.borderColor,
    this.expands = false,
    this.enabled = true,
    this.autocorrect = false,
    this.errorText,
    this.maxLength,
  });

  final String title;
  final String? initial;
  final TextEditingController? controller;
  final Function(String) onChanged;
  final String? Function(String?)? validator;

  /// Use `IconButton` in most cases.
  final Widget? leadingButton;
  final Widget? trailingButton;
  final Widget? prefixIcon;
  final Widget? suffixIcon;
  final bool obscureText;
  final TextInputType? keyboardType;
  final EdgeInsetsGeometry padding;
  final TextStyle? style;
  final TextStyle? titleStyle;
  final int? minLines;
  final int? maxLines;
  final double? radius;
  final Color? borderColor;
  final bool expands;
  final bool enabled;
  final bool autocorrect;
  final String? errorText;
  final int? maxLength;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: padding,
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          leadingButton ?? SizedBox.fromSize(),
          Expanded(
            child: TextFormField(
              enabled: enabled,
              style: style,
              decoration: InputDecoration(
                border: OutlineInputBorder(
                  borderSide: BorderSide(
                    color: borderColor ??
                        BoxWConfig.borderColor ??
                        Theme.of(context).colorScheme.outline,
                  ),
                  borderRadius: BorderRadius.circular(
                    radius ?? BoxWConfig.radius,
                  ),
                ),
                label: Text(title, style: titleStyle),
                errorText: errorText,
                isDense: true,
                prefixIcon: prefixIcon,
                suffixIcon: suffixIcon,
              ),
              controller: controller,
              initialValue: controller == null ? initial : null,
              onChanged: onChanged,
              validator: validator,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              obscureText: obscureText,
              keyboardType: keyboardType,
              minLines: minLines,
              maxLines: obscureText ? 1 : maxLines,
              expands: expands,
              autocorrect: autocorrect,
              maxLength: maxLength,
            ),
          ),
          trailingButton ?? SizedBox.fromSize(),
        ],
      ),
    );
  }
}

class BoxWDropdown<T> extends StatelessWidget {
  const BoxWDropdown({
    Key? key,
    required this.items,
    required this.onChanged,
    required this.value,
    this.title,
    this.borderColor,
    this.radius,
    this.titleStyle,
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
  }) : super(key: key);

  final String? title;
  final T value;
  final List<DropdownMenuItem<T>> items;
  final void Function(T? value)? onChanged;
  final TextStyle? titleStyle;
  final double? radius;
  final Color? borderColor;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: padding,
      child: DropdownButtonFormField<T>(
        decoration: InputDecoration(
          isDense: true,
          border: OutlineInputBorder(
            borderSide: BorderSide(
              color: borderColor ??
                  BoxWConfig.borderColor ??
                  Theme.of(context).colorScheme.outline,
            ),
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
          ),
          label: title == null ? null : Text(title!, style: titleStyle),
        ),
        value: value,
        items: items,
        onChanged: onChanged,
      ),
    );
  }
}

class BoxWButton extends StatelessWidget {
  const BoxWButton({
    Key? key,
    required this.title,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 10, horizontal: 4),
  }) : super(key: key);

  final Widget title;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: ElevatedButton(
        onPressed: onPressed,
        style: ElevatedButton.styleFrom(
          backgroundColor: backgroundColor,
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
            side: BorderSide(
              color: borderColor ??
                  BoxWConfig.borderColor ??
                  Theme.of(context).colorScheme.outline,
            ),
          ),
        ),
        child: Padding(
          padding: padding,
          child: title,
        ),
      ),
    );
  }
}

class BoxWOutlinedButton extends StatelessWidget {
  const BoxWOutlinedButton({
    Key? key,
    required this.title,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 10, horizontal: 4),
  }) : super(key: key);

  final Widget title;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: OutlinedButton(
        onPressed: onPressed,
        style: OutlinedButton.styleFrom(
          backgroundColor: backgroundColor,
          side: BorderSide(
            color: borderColor ??
                BoxWConfig.borderColor ??
                Theme.of(context).colorScheme.outline,
          ),
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
          ),
        ),
        child: Padding(
          padding: padding,
          child: title,
        ),
      ),
    );
  }
}

class BoxWTextButton extends StatelessWidget {
  const BoxWTextButton({
    Key? key,
    required this.title,
    this.onPressed,
    this.backgroundColor,
    this.borderColor,
    this.radius,
    this.width,
    this.height,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 0, horizontal: 12),
  }) : super(key: key);

  final Widget title;
  final Color? backgroundColor;
  final Color? borderColor;
  final VoidCallback? onPressed;
  final double? radius;
  final double? width;
  final double? height;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;

  @override
  Widget build(BuildContext context) {
    return Container(
      margin: margin,
      width: width ?? double.infinity,
      height: height,
      child: TextButton(
        onPressed: onPressed,
        style: TextButton.styleFrom(
          backgroundColor: backgroundColor,
        ),
        child: Padding(
          padding: padding,
          child: title,
        ),
      ),
    );
  }
}

class BoxWText extends StatelessWidget {
  const BoxWText({
    Key? key,
    required this.child,
    this.borderColor,
    this.backgroundColor,
    this.margin = const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
    this.padding = const EdgeInsets.symmetric(vertical: 8, horizontal: 12),
    this.alignment = Alignment.centerLeft,
    this.radius,
    this.width,
    this.height,
  }) : super(key: key);

  final Widget child;
  final Color? borderColor;
  final Color? backgroundColor;
  final EdgeInsetsGeometry margin;
  final EdgeInsetsGeometry padding;
  final AlignmentGeometry alignment;
  final double? radius;
  final double? width;
  final double? height;

  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: alignment,
      width: width ?? double.infinity,
      height: height,
      margin: margin,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.circular(radius ?? BoxWConfig.radius),
        border: Border.all(
          color: borderColor ??
              BoxWConfig.borderColor ??
              Theme.of(context).colorScheme.outline,
        ),
        color: backgroundColor,
      ),
      child: Padding(
        padding: padding,
        child: child,
      ),
    );
  }
}

class BoxWCard extends StatelessWidget {
  const BoxWCard({
    super.key,
    required this.child,
    this.margin,
    this.borderColor,
    this.backgroundColor,
    this.radius,
  });

  final Widget child;
  final EdgeInsetsGeometry? margin;
  final Color? borderColor;
  final Color? backgroundColor;
  final double? radius;

  @override
  Widget build(BuildContext context) {
    return Card(
      elevation: 0,
      color: backgroundColor ?? Theme.of(context).colorScheme.surfaceVariant,
      margin: margin,
      shape: RoundedRectangleBorder(
        side: BorderSide(
          color: borderColor ??
              BoxWConfig.borderColor ??
              Theme.of(context).colorScheme.outline,
        ),
        borderRadius:
            BorderRadius.all(Radius.circular(radius ?? BoxWConfig.radius)),
      ),
      child: child,
    );
  }
}
